UNIT DigBar;

interface

uses
  WinTypes, WinProcs, Messages, SysUtils, Classes, Controls, Forms, Menus, Graphics, Dialogs;

Type
 StyleType = (stNone,stLowered,stRaised);
 TDigBar = class(TGraphicControl)
 private
   FProgress: Integer;
   FBackColor: TColor;
   FSegColor: TColor;
   FInactiveColor: TColor;
   FSegWidth: Integer;
   FSegGap: Integer;
   FEdgeStyle: StyleType;
   FBorder: Integer;
   procedure Initialize;
   procedure SetProgress(Value : Integer);
   procedure SetBackColor(value:Tcolor);
   procedure SetSegColor(value:Tcolor);
   procedure SetInactiveColor(value:Tcolor);
   procedure SetSegWidth(value:Integer);
   procedure SetBorder(value:integer);
   procedure SetSegGap(value:integer);
   procedure SetStyle(value:StyleType);
 protected
   procedure UpdateProgress;
   procedure Paint; Override;
 public
   constructor Create(AOwner: TComponent); override;
   property Canvas;
 published
   property OnClick;
   property OnDblClick;
   property OnMouseDown;
   property OnMouseMove;
   property OnMouseUp;
   property Cursor;
   property EdgeStyle: StyleType read FEdgeStyle write SetStyle default stLowered;
   property BorderWidth: Integer read FBorder write SetBorder default 2;
   property SegmentGap: Integer read FSegGap write SetSegGap default 2;
   property SegmentWidth: Integer read FSegWidth write SetSegWidth default 8;
   property SegmentColor: TColor read FSegColor write SetSegColor default clNavy;
   property InactiveColor: TColor read FInactiveColor write SetInactiveColor default clNone;
   property BackGroundColor:TColor read FBackColor write SetBackColor default clBtnFace;
   property Progress: Integer read FProgress write SetProgress default 0;
   property Width default 100;
   property Height default 18;
 end;

procedure Register;

implementation

procedure TDigBar.SetBorder(Value: Integer);
begin
  if value <> FBorder then
  begin
    FBorder := value;
    Invalidate;
  end;
end;

procedure TDigBar.SetSegWidth(Value: Integer);
begin
  if (value > 0) and (value <> FSegWidth) then
  begin
    FSegWidth := value;
    Invalidate;
  end;
end;

procedure TDigBar.SetSegGap(value:integer);
begin
  if (Value > 0) and (Value <> FSegGap) then
  begin
    FSegGap := Value;
    Invalidate;
  end;
end;

procedure TDigBar.SetBackColor(value:Tcolor);
begin
  if FBackColor <> value then
  begin
    FBackColor := value;
    Invalidate;
  end;
end;

procedure TDigBar.SetSegColor(value:Tcolor);
begin
  if FSegColor <> Value then
  begin
    FSegColor := Value;
    Invalidate;
  end;
end;

procedure TDigBar.SetInactiveColor(value:Tcolor);
begin
  if FInactiveColor <> Value then
  begin
    FInactiveColor := Value;
    Invalidate;
  end;
end;

procedure TDigBar.SetProgress(Value : Integer);
var
  bRefresh : boolean;
begin
  if Value <> FProgress then
  begin
    if FProgress > Value then
      bRefresh := true
    else
      bRefresh := false;
    FProgress := Value;
    if FProgress < 0 then FProgress := 0 else
      if FProgress > 100 then FProgress := 100;
    if (FProgress = 0) or (bRefresh = true) or (csDesigning in ComponentState) then
      Paint  // Invalidate takes longer
    else
      UpdateProgress;
   end;
end;

procedure TDigBar.SetStyle(value:StyleType);
begin
 if value <> FEdgeStyle then
   begin
     FEdgeStyle := value;
     Invalidate;
   end;
end;

procedure TDigBar.Initialize;
begin
 Width := 100;
 Height := 18;
 FProgress := 0;
 FBackColor := clBtnFace;
 FSegColor := clNavy;
 FInactiveColor := clNone;
 FSegWidth := 8;
 FSegGap := 2;
 FBorder := 2;
 FEdgeStyle := stLowered;
end;

constructor TDigBar.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 Initialize;
end;

procedure TDigBar.UpdateProgress;
var
 x1,y1,x2,y2,max, val : integer;
 bg : Tcolor;

procedure DoHorizontal;
var
  i,m : integer;
begin
  m := FBorder+2;
  x1 := m;
  x2 := FSegWidth+m;
  y1 := m;
  y2 := Height-m;
  max := Width div (FSegWidth+FSegGap);
  val := (max * FProgress) div 100;
  for i := 1 to val do
    with Canvas do
    begin
      pen.color := FSegColor;
      brush.color := FSegColor;
      if x2 <= width - m then Rectangle(x1,y1,x2,y2);
      x1 := x1+FSegWidth+FSegGap;
      x2 := x1+FSegWidth;
    end;
  if (FInactiveColor<>clNone) and (val < max) then
    for i := val+1 to max do
      with Canvas do
      begin
        pen.color := FInactiveColor;
        brush.color := FInactiveColor;
        if x2 <= width - m then Rectangle(x1,y1,x2,y2);
        x1 := x1+FSegWidth+FSegGap;
        x2 := x1+FSegWidth;
      end;
end;

procedure DoVertical;
var
   i,h,m : integer;
begin
  h := height;
  m := FBorder+2;
  x1 := m;
  x2 := Width-m;
  y1 := Height-(FSegWidth+m);
  y2 := Height-m;
  max := Height div (FSegWidth+FSegGap);
  val := (max * FProgress) div 100;
  for i := 1 to val do
    with Canvas do
    begin
      pen.color := FSegColor;
      brush.color := FSegColor;
      if y1 >= m then Rectangle(x1,y1,x2,y2);
      y1 := y1-(FSegWidth+FSegGap);
      y2 := y1+ FsegWidth;
    end;
  if (FInactiveColor<>clNone) and (val < max) then
    for i := val+1 to max do
      with Canvas do
      begin
        pen.color := FInactiveColor;
        brush.color := FInactiveColor;
        if y1 >= m then Rectangle(x1,y1,x2,y2);
        y1 := y1-(FSegWidth+FSegGap);
        y2 := y1+ FsegWidth;
      end;
end;

begin
  if Width > Height then DoHorizontal else DoVertical;
end;

procedure TDigBar.Paint;
begin
 with Canvas do
   begin
    Brush.Color := FBackColor;
    if FEdgeStyle = stnone then
    begin
      Pen.Width := 0;
      Pen.Color := FBackColor;
      Rectangle(0,0,width,height);
    end
    else
    begin
      pen.Width := 2;
      if FEdgeStyle = stLowered then pen.color := clgray else
        pen.color := clWhite;
      moveto(0,height);
      lineto(0,0);
      lineto(width-1,0);
      if FEdgeStyle = stLowered then pen.color:=clWhite else
        pen.color := clGray;
      lineto(width-1,height-1);
      lineto(0,height-1);
      Pen.Width := 0;
      Brush.Color :=  FBackColor;
      Pen.Color := FBackColor;;
      Rectangle(1, 1, Width-1, Height-1);
    end;
    UpdateProgress;
   end;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TDigBar]);
end;

end.

