unit Main;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, Menus,
  StdCtrls, Dialogs, Buttons, Messages, ExtCtrls, ComCtrls;

type
  TMainForm = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    FileNewItem: TMenuItem;
    FileOpenItem: TMenuItem;
    FileCloseItem: TMenuItem;
    Window1: TMenuItem;
    Help1: TMenuItem;
    N1: TMenuItem;
    FileExitItem: TMenuItem;
    WindowCascadeItem: TMenuItem;
    WindowTileItem: TMenuItem;
    WindowArrangeItem: TMenuItem;
    HelpAboutItem: TMenuItem;
    OpenDialog: TOpenDialog;
    FileSaveItem: TMenuItem;
    FileSaveAsItem: TMenuItem;
    Edit1: TMenuItem;
    CutItem: TMenuItem;
    CopyItem: TMenuItem;
    PasteItem: TMenuItem;
    WindowMinimizeItem: TMenuItem;
    SpeedPanel: TPanel;
    OpenBtn: TSpeedButton;
    SaveBtn: TSpeedButton;
    CutBtn: TSpeedButton;
    CopyBtn: TSpeedButton;
    PasteBtn: TSpeedButton;
    ExitBtn: TSpeedButton;
    StatusBar: TStatusBar;
    procedure FormCreate(Sender: TObject);
    procedure FileNewItemClick(Sender: TObject);
    procedure WindowCascadeItemClick(Sender: TObject);
    procedure UpdateMenuItems(Sender: TObject);
    procedure WindowTileItemClick(Sender: TObject);
    procedure WindowArrangeItemClick(Sender: TObject);
    procedure FileCloseItemClick(Sender: TObject);
    procedure FileOpenItemClick(Sender: TObject);
    procedure FileExitItemClick(Sender: TObject);
    procedure FileSaveItemClick(Sender: TObject);
    procedure FileSaveAsItemClick(Sender: TObject);
    procedure CutItemClick(Sender: TObject);
    procedure CopyItemClick(Sender: TObject);
    procedure PasteItemClick(Sender: TObject);
    procedure WindowMinimizeItemClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure HelpAboutItemClick(Sender: TObject);
  private
    { Private declarations }
    procedure CreateMDIChild(const Name: string);
    procedure ShowHint(Sender: TObject);
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses ChildWin, About;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Application.OnHint := ShowHint;
  Screen.OnActiveFormChange := UpdateMenuItems;
end;

procedure TMainForm.ShowHint(Sender: TObject);
begin
  StatusBar.SimpleText := Application.Hint;
end;

procedure TMainForm.CreateMDIChild(const Name: string);
var
  Child: TMDIChild;
begin
  { create a new MDI child window }
  Child := TMDIChild.Create(Application);
  Child.Caption := Name;
end;

procedure TMainForm.FileNewItemClick(Sender: TObject);
begin
  CreateMDIChild('NONAME' + IntToStr(MDIChildCount + 1));
end;

procedure TMainForm.FileOpenItemClick(Sender: TObject);
begin
  if OpenDialog.Execute then
    CreateMDIChild(OpenDialog.FileName);
end;

procedure TMainForm.FileCloseItemClick(Sender: TObject);
begin
  if ActiveMDIChild <> nil then
    ActiveMDIChild.Close;
end;

procedure TMainForm.FileSaveItemClick(Sender: TObject);
begin
  { save current file (ActiveMDIChild points to the window) }
end;

procedure TMainForm.FileSaveAsItemClick(Sender: TObject);
begin
  { save current file under new name }
end;

procedure TMainForm.FileExitItemClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.CutItemClick(Sender: TObject);
begin
  {cut selection to clipboard}
end;

procedure TMainForm.CopyItemClick(Sender: TObject);
begin
  {copy selection to clipboard}
end;

procedure TMainForm.PasteItemClick(Sender: TObject);
begin
  {paste from clipboard}
end;

procedure TMainForm.WindowCascadeItemClick(Sender: TObject);
begin
  Cascade;
end;

procedure TMainForm.WindowTileItemClick(Sender: TObject);
begin
  Tile;
end;

procedure TMainForm.WindowArrangeItemClick(Sender: TObject);
begin
  ArrangeIcons;
end;

procedure TMainForm.WindowMinimizeItemClick(Sender: TObject);
var
  I: Integer;
begin
  { Must be done backwards through the MDIChildren array }
  for I := MDIChildCount - 1 downto 0 do
    MDIChildren[I].WindowState := wsMinimized;
end;

procedure TMainForm.UpdateMenuItems(Sender: TObject);
begin
  FileCloseItem.Enabled := MDIChildCount > 0;
  FileSaveItem.Enabled := MDIChildCount > 0;
  FileSaveAsItem.Enabled := MDIChildCount > 0;
  CutItem.Enabled := MDIChildCount > 0;
  CopyItem.Enabled := MDIChildCount > 0;
  PasteItem.Enabled := MDIChildCount > 0;
  SaveBtn.Enabled := MDIChildCount > 0;
  CutBtn.Enabled := MDIChildCount > 0;
  CopyBtn.Enabled := MDIChildCount > 0;
  PasteBtn.Enabled := MDIChildCount > 0;
  WindowCascadeItem.Enabled := MDIChildCount > 0;
  WindowTileItem.Enabled := MDIChildCount > 0;
  WindowArrangeItem.Enabled := MDIChildCount > 0;
  WindowMinimizeItem.Enabled := MDIChildCount > 0;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  Screen.OnActiveFormChange := nil;
end;

procedure TMainForm.HelpAboutItemClick(Sender: TObject);
begin
  AboutBox.ShowModal;
end;

end.
unit TestMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, PSK31Core, ComCtrls, Buttons, Menus, Clipbrd, DigBar,
  Anaclock, AnalogMeter;

type
  TUA4HVS_PSK31 = class(TForm)
    StatusBar: TStatusBar;
    Timer: TTimer;
    RxPanel: TGroupBox;
    GroupBox1: TGroupBox;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Call: TEdit;
    Name: TEdit;
    RST: TEdit;
    ClearBtn: TButton;
    GroupBox2: TGroupBox;
    GroupBox3: TGroupBox;
    M1: TButton;
    M6: TButton;
    M2: TButton;
    M7: TButton;
    M3: TButton;
    M8: TButton;
    M4: TButton;
    M9: TButton;
    M5: TButton;
    M10: TButton;
    ImageList1: TImageList;
    TerminalPopup: TPopupMenu;
    ClearTerminal: TMenuItem;
    SndVolPopup: TPopupMenu;
    RxAdjMenu: TMenuItem;
    TxAdjMenu: TMenuItem;
    CopyTerminal: TMenuItem;
    TxBuf: TMemo;
    TxBufPopup: TPopupMenu;
    ClearTXBuf: TMenuItem;
    PastetoTX: TMenuItem;
    Terminal: TMemo;
    AnalogClock1: TAnalogClock;
    SpectrumDisplay: TPaintBox;
    WaterfallDisplay: TPaintBox;
    Mode: TRadioGroup;
    S_Meter: TDigBar;
    MeterPanel: TPanel;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    SquelchInd: TLabel;
    Onbtn: TSpeedButton;
    TuneBtn: TSpeedButton;
    TxBtn: TSpeedButton;
    RxFreq: TEdit;
    TxFreq: TEdit;
    RxFreqDown: TButton;
    TxFreqDown: TButton;
    RxFreqUp: TButton;
    TxFreqUp: TButton;
    AFC: TCheckBox;
    NET: TCheckBox;
    IMD: TLabel;
    Label6: TLabel;
    Button1: TButton;
    MainMenu1: TMainMenu;
    procedure FormCreate(Sender: TObject);
    procedure OnBtnClick(Sender: TObject);
    procedure SpectrumDisplayPaint(Sender: TObject);
    procedure AFCClick(Sender: TObject);
    procedure RxFreqDownClick(Sender: TObject);
    procedure RxFreqUpClick(Sender: TObject);
    procedure WaterfallDisplayMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure WaterfallDisplayPaint(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure TuneBtnClick(Sender: TObject);
    procedure TxBtnClick(Sender: TObject);
    procedure ModeClick(Sender: TObject);
    procedure TxFreqDownClick(Sender: TObject);
    procedure TxFreqUpClick(Sender: TObject);
    procedure ClearBtnClick(Sender: TObject);
    procedure TxBufKeyPress(Sender: TObject; var Key: Char);
    procedure MacroClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure StatusBarDrawPanel(StatusBar: TStatusBar;
      Panel: TStatusPanel; const Rect: TRect);
    procedure StatusBarMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ClearTerminalClick(Sender: TObject);
    procedure RxAdjMenuClick(Sender: TObject);
    procedure TxAdjMenuClick(Sender: TObject);
    procedure CopyTerminalClick(Sender: TObject);
    procedure ClearTXBufClick(Sender: TObject);
    procedure PastetoTXClick(Sender: TObject);
    procedure TxBufPopupPopup(Sender: TObject);
    procedure MeterPanelMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Button1Click(Sender: TObject);
  protected
    procedure FFTDataReady(var msg: TMessage); message MSG_DATARDY;
    procedure CharReady(var msg: TMessage); message MSG_PSKCHARRDY;
    procedure IMDReady(var msg: TMessage); message MSG_IMDRDY;
    procedure ClkErr(var msg: TMessage); message MSG_CLKERR;
    procedure StatusChange(var msg: TMessage); message MSG_STATUSCHANGE;
  private
    { Private declarations }
    procedure LoadMacros;
  public
    { Public declarations }
  end;

var
  UA4HVS_PSK31: TUA4HVS_PSK31;

implementation

{$R *.DFM}

uses Utils;

var
	AppEnabled: boolean = false;
	SignalOverload: boolean;
  WaterfallBitmap: TBitmap;
	FFTData: TFFTData;
  FFTMax, FFTScale, FFTBandwidth: integer;
  WFMin, WFMax, WFBandwidth, WFOffset: integer;
  RXFrequency, TXFrequency: integer;

  inifile: string;

  showutctime: boolean;
  centerfreq, afcrange: integer;
  mycall, dtformat: string;

  macro: array[1..10] of string;

const
	sSettings = 'Settings';
  sDisplay = 'Display';
  sTxCtrl = 'TX Control';
  sPersistent = 'Persistent';


procedure TUA4HVS_PSK31.FormCreate(Sender: TObject);
var
	s: string;
  comport,control,thresh: integer;
begin
  // create waterfall bitmap
  with WaterfallDisplay do
  begin
	  WaterfallBitmap := TBitmap.Create;
  	WaterfallBitmap.Width := Width;
	  WaterfallBitmap.Height := Height;
  end;
  // read settings from ini file
  inifile := ChangeFileExt(Application.ExeName,'.ini');
  if not FileExists(inifile) then
		Application.MessageBox(PChar('Cannot open file '+inifile),PChar(Application.Title),mb_IconExclamation+mb_OK);
  mycall := GetStringFromIniFile(inifile,sSettings,'MyCall','CALL NOT SET');
  s := GetStringFromIniFile(inifile,sSettings,'Caption','PSK31 Demo');
 	Caption := s+' - '+mycall;
  showutctime := GetStringFromIniFile(inifile,sSettings,'ShowUTCTime','') = '1';
  dtformat := GetStringFromIniFile(inifile,sSettings,'DateFormat','c');
  fnSetCWIDSpeed(StrToInt(GetStringFromIniFile(inifile,sSettings,'CWIDSpeed','3')));
  fnSetCWIDString(PChar(GetStringFromIniFile(inifile,sSettings,'CWID','')));
  afcrange := StrToInt(GetStringFromIniFile(inifile,sSettings,'AFCRange','10'));
  comport := StrToInt(GetStringFromIniFile(inifile,sTxCtrl,'ComPort','1'));
  control := StrToInt(GetStringFromIniFile(inifile,sTxCtrl,'Control','0'));
  // set center frequency = starting Rx and Tx frequency
  RxFreq.Text := GetStringFromIniFile(inifile,sDisplay,'CenterFreq','1000');
  TxFreq.Text := RxFreq.Text;
  centerfreq := StrToInt(RxFreq.Text);
  RxFrequency := centerfreq;
  TxFrequency := centerfreq;
	// set FFT parameters
	FFTMax := centerfreq * 2048 div 4000 - 1;
  if FFTMax > 1023 then FFTMax := 1023;
  FFTScale := 100;
  FFTBandwidth := (FFTMax+1) * 8000 div 2048;
  // set squelch threshold and other persistent settings
  thresh := StrToInt(GetStringFromIniFile(inifile,sPersistent,'Squelch Level','10'));
  fnSetSquelchThreshold(thresh,1,0);
  SquelchInd.Left := thresh;
	AFC.Checked := GetStringFromIniFile(inifile,sPersistent,'AFC','1')='1';
	Net.Checked := GetStringFromIniFile(inifile,sPersistent,'Net','1')='1';
	// set waterfall display parameters
  with WaterfallDisplay do
  begin
		WFMin := (FFTMax div 2) - Width div 2;
    WFMax := (FFTMax div 2) + Width div 2 - 1;
    WFBandwidth := (WFMax - WFMin) * 8000 div 2048;
  	WFOffset := WFMin * 8000 div 2048;
  end;
  // set TX control port
  if control <> 0 then
  	if not fnSetComPort(comport,control) then
    	Application.MessageBox('Error in COM port selection for PTT control',PChar(Application.Title),mb_IconExclamation+mb_OK);
	// display DLL version
 	s := IntToStr(fnGetDLLVersion);
  Insert('.',s,2);
  AppEnabled := OnBtn.Down;
  StatusBar.Panels[3].Text := 'PSKCORE.DLL version '+s;
  // load macro key text
  LoadMacros;
end;

procedure TUA4HVS_PSK31.FormShow(Sender: TObject);
begin
	// make small edit boxes
  RXFreq.Height := 17;
  TXFreq.Height := 17;
  Call.Height := 18;
  Name.Height := 18;
  RST.Height := 18;
end;

procedure TUA4HVS_PSK31.FormDestroy(Sender: TObject);
begin
	WriteStringToIniFile(inifile,sPersistent,'Squelch Level',IntToStr(SquelchInd.Left));
	WriteStringToIniFile(inifile,sPersistent,'AFC',IntToStr(Ord(AFC.Checked)));
	WriteStringToIniFile(inifile,sPersistent,'Net',IntToStr(Ord(Net.Checked)));
	if AppEnabled then fnStopSoundCard;
  WaterfallBitmap.Free;
end;

procedure TUA4HVS_PSK31.OnBtnClick(Sender: TObject);
var
	err: integer;
  errstr: string;
  Rect: TRect;
begin
	AppEnabled := OnBtn.Down;
  if AppEnabled then
  begin
   	err := fnStartSoundCard(Handle,-1,1);
    if err = 0 then
    begin
		  // initialise DLL settings
			fnSetFFTMode(1, FFTScale, 1);
      fnSetRXPSKMode(Mode.ItemIndex,0);
		  fnSetAFCLimit(Ord(AFC.Checked)*AFCRange,0);
			fnSetSquelchThreshold(15,1,0);
      fnSetRXFrequency(RxFrequency, 0, 0);
      with WaterfallBitmap do
      begin
			  Rect.Left := 0;
		  	Rect.Top := 0;
			  Rect.Right := Width;
		  	Rect.Bottom := Height;
		    with WaterfallBitmap.Canvas do
		    begin
  			  Brush.Color := clBlack;
			  	FillRect(Rect);
		    end;
      end;
      OnBtn.Font.Color := clRed;
      TuneBtn.Enabled := true;
      TxBtn.Enabled := true;
      M1.Enabled := Macro[1] <> '';
      M2.Enabled := Macro[2] <> '';
      M3.Enabled := Macro[3] <> '';
      M4.Enabled := Macro[4] <> '';
      M5.Enabled := Macro[5] <> '';
      M6.Enabled := Macro[6] <> '';
      M7.Enabled := Macro[7] <> '';
      M8.Enabled := Macro[8] <> '';
      M9.Enabled := Macro[9] <> '';
      M10.Enabled := Macro[10] <> '';
    end
    else
    begin
    	case err of
      10: errstr := 'Memory allocation error';
      11: errstr := 'Cannot open sound card';
      12: errstr := 'Sound card failure';
      13: errstr := 'Sound card already in use';
      else
      	errstr := 'Undefined error: code '+IntToStr(err);
      end;
      Application.MessageBox(PChar(errstr),PChar(Application.Title),mb_IconExclamation+mb_OK);
    end;
  end
  else
  begin
  	if TuneBtn.Down then
    begin
    	TuneBtn.Down := false;
	  	fnStopTX;
  	  TuneBtn.Font.Color := clWindowText;
      Delay(2);
    end;
  	if TxBtn.Down then
    begin
    	TxBtn.Down := false;
      if fnGetTXCharsRemaining = 0 then
		    fnClearTxBuffer;
	  	fnAbortTX;
  	  TxBtn.Font.Color := clWindowText;
      Delay(2);
    end;
    TxBuf.Clear;
   	fnStopSoundCard;
    SpectrumDisplayPaint(Sender);
    WaterfallDisplayPaint(Sender);
    OnBtn.Font.Color := clWindowText;
    TuneBtn.Enabled := false;
    TxBtn.Enabled := false;
    M1.Enabled := false;
    M2.Enabled := false;
    M3.Enabled := false;
    M4.Enabled := false;
    M5.Enabled := false;
    M6.Enabled := false;
    M7.Enabled := false;
    M8.Enabled := false;
    M9.Enabled := false;
    M10.Enabled := false;
  end;
end;

procedure TUA4HVS_PSK31.FFTDataReady(var msg: TMessage);
begin
	if AppEnabled then
  begin
	 	if fnGetFFTData( @FFTData, 0, FFTMax ) <> SignalOverload then
    begin
    	StatusBar.Panels[2].Text := '** Input Overload **';
      SignalOverload := true;
    end
    else
    begin
    	StatusBar.Panels[2].Text := 'Receive';
      SignalOverload := false;
    end;
    S_Meter.Progress := msg.LParam;
    RxFrequency := msg.WParam;
    RxFreq.Text := IntToStr(RxFrequency);
  	SpectrumDisplayPaint(self);
  	WaterfallDisplayPaint(self);
  end;
end;

procedure TUA4HVS_PSK31.CharReady(var msg: TMessage);
var
	c: Word;
begin
	if AppEnabled then
  begin
  	c := msg.WParam;
    if c in [8,13,32..255] then
    begin
			Terminal.Perform(EM_SETSEL,30000,30000);
    	Terminal.Perform(WM_CHAR,c,0);
    end;
  end;
end;

procedure TUA4HVS_PSK31.IMDReady(var msg: TMessage);
begin
	if AppEnabled then
  begin
  	if msg.LParam < 100 then
    	IMD.Caption := IntToStr(msg.wParam)+'dB'
    else
    	IMD.Caption := '';
  end;
end;

procedure TUA4HVS_PSK31.ClkErr(var msg: TMessage);
begin
	if AppEnabled then
  	if msg.lParam = 0 then
    	StatusBar.Panels[1].Text := ''
    else
  		StatusBar.Panels[1].Text := 'Clk Err: '+IntToStr(msg.lParam)+'ppm';
end;

procedure TUA4HVS_PSK31.StatusChange(var msg: TMessage);
var
	errstr: string;
begin
	errstr := '';
	if AppEnabled then
  begin
  	case msg.wParam of
    0:	begin
    			StatusBar.Panels[2].Text := 'Receive';
          if TxBtn.Down then
          begin
          	TxBtn.Down := false;
			  	  TxBtn.Font.Color := clWindowText;
				    TuneBtn.Enabled := true;
            TxBuf.Clear;
          end;
    		end;
    1:	StatusBar.Panels[2].Text := 'Transmit!';
    2:	StatusBar.Panels[2].Text := '** CPU busy **';
    10: errstr := 'Memory allocation error';
    11: errstr := 'Cannot open sound card';
    12: errstr := 'Sound card failure';
    13: errstr := 'Sound card already in use';
    else
    	errstr := 'Undefined error: code '+IntToStr(msg.wParam);
    end;
    if errstr <> '' then
      Application.MessageBox(PChar(errstr),PChar(Application.Title),mb_IconExclamation+mb_OK);
  end;
end;

procedure TUA4HVS_PSK31.SpectrumDisplayPaint(Sender: TObject);
var
  DisplayBitmap: TBitmap;
  DisplayRect: TRect;
  bmwidth, bmheight, i, fftrange: Integer;
begin
  with SpectrumDisplay do
  begin
		bmwidth := Width;
  	bmheight := Height;
	  DisplayBitmap := TBitmap.Create;
  	DisplayBitmap.Width := bmwidth;
	  DisplayBitmap.Height := bmheight;
	  DisplayRect.Left := 0;
  	DisplayRect.Top := 0;
	  DisplayRect.Right := bmwidth;
  	DisplayRect.Bottom := bmheight;
    with DisplayBitmap.Canvas do
    begin
  	  Brush.Color := clBlack;
	  	FillRect(DisplayRect);
    end;
    if AppEnabled then
 	  begin
      (* == this code displays a spectrum analyzer display == *)
		  with DisplayBitmap.Canvas do
    	begin
	  	  // draw Rx frequency position
	    	Pen.Color := clRed;
		    i := RxFrequency * bmwidth div FFTBandwidth;
  		  MoveTo(i,0);
    		LineTo(i,bmheight-10);
				// now draw spectrum display
  	  	Pen.Color := clAqua;
        MoveTo(0, bmheight-1);
	  		LineTo(0, bmheight-(FFTData[0]*bmheight) div FFTScale);
	 	  	fftrange := FFTMax;
 		  	for i := 1 to bmwidth do
        begin
        	MoveTo(i, bmheight-1);
		  	  LineTo(i, bmheight-(FFTData[(i*fftrange div bmwidth)]*bmheight) div FFTScale);
        end;
      end;
    end;
		BitBlt(Canvas.Handle, 0, 0, Width, Height, DisplayBitmap.Canvas.Handle, 0, 0, SRCCOPY);
	  DisplayBitmap.Free;
  end;
end;

procedure TUA4HVS_PSK31.WaterfallDisplayMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  with WaterfallDisplay do
  begin
  	if Button = mbLeft then
    begin
			RXFrequency := (X+1) * WFBandwidth div Width + WFOffset - 2;
  		fnSetRXFrequency(RXFrequency, 0, 0);
    end;
    if not AFC.Checked then
    begin
    	fnSetAFCLimit(afcrange,0);
      Delay(10);
      fnSetAFCLimit(0, 0);
      RXFrequency := fnGetRXFrequency(0);
    end;
  	RxFreq.Text := IntToStr(RXFrequency);
  end;
end;

procedure TUA4HVS_PSK31.WaterfallDisplayPaint(Sender: TObject);
var
  TempBitmap: TBitmap;
  Rect: TRect;
  bmwidth, bmheight, i, v, g: Integer;
begin
  with WaterfallDisplay do
  begin
		bmwidth := Width;
  	bmheight := Height;
	  TempBitmap := TBitmap.Create;
  	TempBitmap.Width := bmwidth;
	  TempBitmap.Height := bmheight;
	  Rect.Left := 0;
  	Rect.Top := 0;
	  Rect.Right := bmwidth;
  	Rect.Bottom := bmheight;
    if AppEnabled then
 	  begin
      (* == this code draws a waterfall display == *)
		  with TempBitmap.Canvas do
    	begin
      	// copy previous waterfall bitmap shifted down 1 pixel
      	BitBlt(Handle,0,1,bmwidth,bmheight-1,WaterfallBitmap.Canvas.Handle,0,0,SRCCOPY);
       // write new waterfall data
        for i := 0 to bmwidth-1 do
        begin
        	v := FFTData[WFMin+i];
          g := v shl 2; if g > 255 then g := 255;
          TempBitmap.Canvas.Pixels[i,0] := RGB(v,g,v);
        end;
        // copy it back to waterfall bitmap
      	BitBlt(WaterfallBitmap.Canvas.Handle,0,0,bmwidth,bmheight,Handle,0,0,SRCCOPY);
	  	  // draw Rx frequency position
	    	Pen.Color := clRed;
		    i := (RxFrequency - WFOffset) * bmwidth div wfBandwidth;
  		  MoveTo(i,0);
    		LineTo(i,bmheight-1);
				// now draw waterfall display
      end;
    end
    else
	    with TempBitmap.Canvas do
  	  begin
  		  Brush.Color := clBlack;
	  		FillRect(Rect);
	    end;
		BitBlt(Canvas.Handle, 0, 0, Width, Height, TempBitmap.Canvas.Handle, 0, 0, SRCCOPY);
	  TempBitmap.Free;
  end;
end;

procedure TUA4HVS_PSK31.AFCClick(Sender: TObject);
begin
	if AFC.Checked then
  	fnSetAFCLimit(afcrange,0)
  else
  	fnSetAFCLimit(0,0);
end;

procedure TUA4HVS_PSK31.RxFreqDownClick(Sender: TObject);
begin
	RxFrequency := fnGetRXFrequency(0) - 1;
  fnSetRXFrequency(RxFrequency, 0, 0);
  RxFreq.Text := IntToStr(RxFrequency);
end;

procedure TUA4HVS_PSK31.RxFreqUpClick(Sender: TObject);
begin
	RxFrequency := fnGetRXFrequency(0) + 1;
  fnSetRXFrequency(RxFrequency, 0, 0);
  RxFreq.Text := IntToStr(RxFrequency);
end;

procedure TUA4HVS_PSK31.TxFreqDownClick(Sender: TObject);
begin
	TxFrequency := TxFrequency + 1;
  fnSetTXFrequency(TxFrequency);
  TxFreq.Text := IntToStr(TxFrequency);
end;

procedure TUA4HVS_PSK31.TxFreqUpClick(Sender: TObject);
begin
	TxFrequency := TxFrequency - 1;
  fnSetTXFrequency(TxFrequency);
  TxFreq.Text := IntToStr(TxFrequency);
end;

procedure TUA4HVS_PSK31.TimerTimer(Sender: TObject);
var
	t: TDateTime;
begin
	t := Now;
  if showutctime then t := t + TimeZoneBias;
	StatusBar.Panels[0].Text := FormatDateTime('hh:nn:ss',t);
end;

procedure TUA4HVS_PSK31.TuneBtnClick(Sender: TObject);
begin
	if TuneBtn.Down then
  begin
  	TxBtn.Enabled := false;
    if Net.Checked then
    	TxFreq.Text := RxFreq.Text;
    TxFrequency := StrToInt(TxFreq.Text);
    fnSetTXFrequency(TxFrequency);
  	fnStartTX(3);
    TuneBtn.Font.Color := clRed;
  end
  else
  begin
  	fnStopTX;
    TuneBtn.Font.Color := clWindowText;
    TxBtn.Enabled := true;
  end;
end;

procedure TUA4HVS_PSK31.TxBtnClick(Sender: TObject);
begin
	if TxBtn.Down then
  begin
    TuneBtn.Enabled := false;
    if Net.Checked then
    	TxFreq.Text := RxFreq.Text;
    TxFrequency := StrToInt(TxFreq.Text);
    fnSetTXFrequency(TxFrequency);
 		fnStartTX(Mode.ItemIndex);
   	TxBtn.Font.Color := clRed;
  end
  else
  begin
  	if fnGetTXCharsRemaining = 0 then
    	fnStopTX
    else
    begin
  		fnAbortTX;
	    fnClearTXBuffer;
    end;
    TxBuf.Clear;
    TxBtn.Font.Color := clGreen;
    TuneBtn.Enabled := true;
  end;
end;

procedure TUA4HVS_PSK31.ModeClick(Sender: TObject);
begin
  fnSetRXPSKMode(Mode.ItemIndex,0);
end;

procedure TUA4HVS_PSK31.ClearBtnClick(Sender: TObject);
begin
	Call.Text := '';
  Name.Text := '';
  RST.Text := '';
end;

procedure TUA4HVS_PSK31.TxBufKeyPress(Sender: TObject; var Key: Char);
var
	c: Integer;
begin
	TxBuf.Perform(EM_SETSEL,9999,9999);
  Application.ProcessMessages;
	c := Ord(Key);
  if c = 3 then
		fnSendTxCharacter(1,1)
  else if c = 9 then
  	fnSendTxCharacter(2,1)
  else
  	fnSendTxCharacter(c,0);
end;

procedure TUA4HVS_PSK31.LoadMacros;
var
	i: integer;
	s,n: string;
  c: TComponent;
begin
	for i := 1 to 10 do
	begin
  	n := IntToStr(i);
  	s := GetStringFromIniFile(inifile,'Macro '+n,'Caption','');
    if s <> '' then
    begin
    	c := FindComponent('M'+n);
      if c <> nil then
      begin
	      with TButton(c) do
  	    begin
    	  	Caption := s;
      	  Hint := GetStringFromIniFile(inifile,'Macro '+n,'Hint','');
	      end;
  	    Macro[i] := GetStringFromIniFile(inifile,'Macro '+n,'Text','');
      end;
    end;
	end;
end;

procedure TUA4HVS_PSK31.MacroClick(Sender: TObject);
var
	i,p: Integer;
  h,m,ss,ms: Word;
  txt: TStringList;
  s,g,t: string;
begin
  s := TButton(Sender).Name;
  Delete(s,1,1);
  i := StrToInt(s);
  s := Macro[i];
  // process macro tokens
  repeat
	  p := Pos('\c',s);    // \c = their call
    if p > 0 then
    begin
    	Delete(s,p,2);
      if Call.Text = '' then
      begin
      	Call.Text := InputBox(Application.Title,'Enter stations call','');
      end;
      Insert(Call.Text,s,p);
    end;
  until p = 0;
  repeat
	  p := Pos('\d',s);    // \d = date and time
    if p > 0 then
    begin
    	Delete(s,p,2);
      Insert(FormatDateTime(dtformat,Now+TimeZoneBias),s,p);
    end;
  until p = 0;
  repeat
	  p := Pos('\f',s);    // \fFILENAME.TXT = insert text
    if p > 0 then
    begin
    	Delete(s,p,2);
      i := 0;
      repeat
      	inc(i)
      until s[p+i]<#33;
      t := ExtractFilePath(Application.ExeName)+Copy(s,p,i);
      Delete(s,p,i);
      if FileExists(t) then
      begin
      	txt := TStringList.Create;
        txt.LoadFromFile(t);
        t := '';
        if txt.Count > 0 then
        begin
        	for i := 0 to txt.Count - 1 do
          	t := t + txt.Strings[i]+#13;
          Insert(t,s,p);
        end;
        txt.Free;
      end;
    end;
  until p = 0;
  repeat
	  p := Pos('\g',s);    // \g = greeting
    if p > 0 then
    begin
    	Delete(s,p,2);
      DecodeTime(Now,h,m,ss,ms);
      if h < 12 then g := 'Good morning'
      else if h < 18 then g := 'Good afternoon'
      else g := 'Good evening';
      Insert(g,s,p);
    end;
  until p = 0;
  repeat
	  p := Pos('\h',s);    // \h = their name
    if p > 0 then
    begin
    	if Name.Text = '' then
      	Delete(s,p,3)
      else
      begin
    		Delete(s,p,2);
      	Insert(Name.Text,s,p);
      end;
    end;
  until p = 0;
  repeat
	  p := Pos('\i',s);    // \i = CWID
    if p > 0 then
    begin
    	Delete(s,p,2);
      Insert(#9,s,p);
    end;
  until p = 0;
  repeat
	  p := Pos('\m',s);    // \m = my call
    if p > 0 then
    begin
    	Delete(s,p,2);
      Insert(mycall,s,p);
    end;
  until p = 0;
  repeat
	  p := Pos('\n',s);    // \n = newline
    if p > 0 then
    begin
    	Delete(s,p,2);
      Insert(#13,s,p);
    end;
  until p = 0;
  repeat
	  p := Pos('\r',s);    // \r = their report
    if p > 0 then
    begin
    	Delete(s,p,2);
      if RST.Text = '' then
      begin
      	RST.Text := InputBox(Application.Title,'Enter stations report','599');
      end;
      Insert(RST.Text,s,p);
    end;
  until p = 0;
  repeat
	  p := Pos('\t',s);    // \t = start transmit
    if p > 0 then
    begin
    	Delete(s,p,2);
      if not TuneBtn.Down then
      begin
		    if Net.Checked then
    			TxFreq.Text := RxFreq.Text;
		    TxFrequency := StrToInt(TxFreq.Text);
    		fnSetTXFrequency(TxFrequency);
        TxBtn.Down := true;
		 		fnStartTX(Mode.ItemIndex);
   			TxBtn.Font.Color := clRed;
      end;
    end;
  until p = 0;
  repeat
	  p := Pos('\w',s);    // \w = clear terminal window
    if p > 0 then
    begin
    	Delete(s,p,2);
      Terminal.Clear;
    end;
  until p = 0;
  repeat
	  p := Pos('\z',s);    // \z = end Tx
    if p > 0 then
    begin
    	Delete(s,p,2);
      Insert(#3,s,p);
    end;
  until p = 0;
  for i := 1 to Length(s) do
 		TxBuf.Perform(WM_CHAR,Ord(s[i]),0);
end;

procedure TUA4HVS_PSK31.StatusBarDrawPanel(StatusBar: TStatusBar;
  Panel: TStatusPanel; const Rect: TRect);
begin
	with StatusBar.Canvas do
    ImageList1.Draw(StatusBar.Canvas,Rect.Left+2,Rect.Top,0);
end;

procedure TUA4HVS_PSK31.StatusBarMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
	point: TPoint;
begin
	if X > 612 then
  begin
		if Button = mbRight then
    begin
    	GetCursorPos(point);
			SndVolPopup.Popup(point.X,point.Y);
    end
    else
    	WinExec('sndvol32.exe', SW_SHOW);
  end;
end;

procedure TUA4HVS_PSK31.RxAdjMenuClick(Sender: TObject);
begin
  WinExec('sndvol32.exe /r', SW_SHOW);
end;

procedure TUA4HVS_PSK31.TxAdjMenuClick(Sender: TObject);
begin
	WinExec('sndvol32.exe /p',SW_SHOW);
end;

procedure TUA4HVS_PSK31.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbRight then
  	Application.MessageBox(PChar('X: '+IntToStr(X)+'  Y: '+IntToStr(Y)),PChar(Application.Title),mb_IconInformation+mb_OK);
end;

procedure TUA4HVS_PSK31.ClearTerminalClick(Sender: TObject);
begin
  Terminal.Clear;
end;

procedure TUA4HVS_PSK31.CopyTerminalClick(Sender: TObject);
begin
	Clipboard.SetTextBuf(PChar(Terminal.Lines.Text));
end;

procedure TUA4HVS_PSK31.ClearTXBufClick(Sender: TObject);
begin
  fnClearTXBuffer;
  TxBuf.Clear;
end;

procedure TUA4HVS_PSK31.PastetoTXClick(Sender: TObject);
var
	t: string;
	i,c: integer;
begin
	t := Clipboard.AsText;
  for i := 1 to Length(t) do
  begin
  	c := Ord(t[i]);
    if (c > 12) then
 			TxBuf.Perform(WM_CHAR,c,0);
  end;
end;

procedure TUA4HVS_PSK31.TxBufPopupPopup(Sender: TObject);
begin
  PasteToTX.Enabled := Clipboard.HasFormat(CF_TEXT);
end;

procedure TUA4HVS_PSK31.MeterPanelMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
	thresh: integer;
begin
	thresh := X;
  if thresh < 2 then thresh := 2;
  if thresh > 99 then thresh := 99;
  fnSetSquelchThreshold(thresh,1,0);
  SquelchInd.Left := thresh;
end;
procedure TUA4HVS_PSK31.Button1Click(Sender: TObject);
begin
     Close;
end;

end.
